@@ -5,15 +5,23 @@ Most part of this documentation is using Python type hinting.
 """
 
 from abc import ABC
-from typing import AnyStr, Dict, Generator, List, Union, TYPE_CHECKING
+import math
+from typing import AnyStr, Dict, Generator, List, Union, Callable, Tuple, TYPE_CHECKING
 from somelib import SomeInterface
 import zope.interface
 import zope.schema
 from typing import Sequence, Optional
+from incremental import Version
+from twisted.python.deprecate import deprecated, deprecatedProperty
 
 if TYPE_CHECKING:
     from typing_extensions import Final
 
+Parser = Callable[[str], Tuple[int, bytes, bytes]]
+"""
+Type aliases are documented as such and their value is shown just like constants.
+"""
+
 LANG = 'Fr'
 """
 This is a constant. See L{constants} for more examples.
@@ -24,6 +32,10 @@ lang: 'Final[Sequence[str]]' = ['Fr', 'En']
 This is also a constant, but annotated with typing.Final.
 """
 
+@deprecated(Version("demo", "NEXT", 0, 0), replacement=math.prod)
+def demo_product_deprecated(x, y) -> float: # type: ignore
+    return float(x * y)
+
 def demo_fields_docstring_arguments(m, b):  # type: ignore
     """
     Fields are used to describe specific properties of a documented object.
@@ -113,6 +125,13 @@ class DemoClass(ABC, SomeInterface, _PrivateClass):
         """
         return 1
 
+    @deprecatedProperty(Version("demo", 1, 3, 0), replacement=read_only)
+    def read_only_deprecated(self) -> int:
+        """
+        This is a deprecated read-only property.
+        """
+        return 1
+
     @property
     def read_and_write(self) -> int:
         """
