@@ -4,13 +4,22 @@ This is a module demonstrating reST code documentation features.
 Most part of this documentation is using Python type hinting.
 """
 from abc import ABC
+from ast import Tuple
+import math
 import zope.interface
 import zope.schema
-from typing import Sequence, Optional, TYPE_CHECKING
+from typing import Callable, Sequence, Optional, AnyStr, Generator, Union, List, Dict, TYPE_CHECKING
+from incremental import Version
+from twisted.python.deprecate import deprecated, deprecatedProperty
 
 if TYPE_CHECKING:
     from typing_extensions import Final
 
+Parser = Callable[[str], Tuple[int, bytes, bytes]]
+"""
+Type aliases are documented as such and their value is shown just like constants.
+"""
+
 LANG = 'Fr'
 """
 This is a constant. See `constants` for more examples.
@@ -21,9 +30,9 @@ lang: 'Final[Sequence[str]]' = ['Fr', 'En']
 This is also a constant, but annotated with typing.Final.
 """
 
-
-from typing import AnyStr, Generator, Union, List, Dict
-
+@deprecated(Version("demo", "NEXT", 0, 0), replacement=math.prod)
+def demo_product_deprecated(x, y) -> float: # type: ignore
+    return float(x * y)
 
 def demo_fields_docstring_arguments(m, b = 0):  # type: ignore
     """
@@ -133,6 +142,13 @@ class DemoClass(ABC, _PrivateClass):
         """
         return 1
 
+    @deprecatedProperty(Version("demo", 1, 3, 0), replacement=read_only)
+    def read_only_deprecated(self) -> int:
+        """
+        This is a deprecated read-only property.
+        """
+        return 1
+
     @property
     def read_and_write(self) -> int:
         """
