@@ -2,20 +2,22 @@
 
 import ast
 import sys
-from attr import attrs, attrib
+
 from functools import partial
-from inspect import BoundArguments, Parameter, Signature, signature
+from inspect import Parameter, Signature
 from itertools import chain
 from pathlib import Path
 from typing import (
-    Any, Callable, Dict, Iterable, Iterator, List, Mapping, Optional, Sequence, Tuple,
+    Any, Callable, Collection, Dict, Iterable, Iterator, List, Mapping, Optional, Sequence, Tuple,
     Type, TypeVar, Union, cast
 )
 
 import astor
-from pydoctor import epydoc2stan, model, node2stan
+from pydoctor import epydoc2stan, model, node2stan, extensions
 from pydoctor.epydoc.markup._pyval_repr import colorize_inline_pyval
-from pydoctor.astutils import bind_args, node2dottedname, is__name__equals__main__
+from pydoctor.astutils import (is_typing_annotation, is_using_annotations, is_using_typing_final, node2dottedname, node2fullname, 
+                               is__name__equals__main__, unstring_annotation, iterassign, 
+                               NodeVisitor)
 
 def parseFile(path: Path) -> ast.Module:
     """Parse the contents of a Python source file."""
@@ -29,20 +31,13 @@ else:
     _parse = ast.parse
 
 
-def node2fullname(expr: Optional[ast.expr], ctx: model.Documentable) -> Optional[str]:
-    dottedname = node2dottedname(expr)
-    if dottedname is None:
-        return None
-    return ctx.expandName('.'.join(dottedname))
-
-
 def _maybeAttribute(cls: model.Class, name: str) -> bool:
     """Check whether a name is a potential attribute of the given class.
     This is used to prevent an assignment that wraps a method from
     creating an attribute that would overwrite or shadow that method.
 
     @return: L{True} if the name does not exist or is an existing (possibly
-        inherited) attribute, L{False} otherwise
+        inherited) attribute, L{False} if this name defines something else than an L{Attribute}. 
     """
     obj = cls.find(name)
     return obj is None or isinstance(obj, model.Attribute)
@@ -65,100 +60,6 @@ def _handleAliasing(
     ctx._localNameToFullName_map[target] = full_name
     return True
 
-_attrs_decorator_signature = signature(attrs)
-"""Signature of the L{attr.s} class decorator."""
-
-def _uses_auto_attribs(call: ast.Call, module: model.Module) -> bool:
-    """Does the given L{attr.s()} decoration contain C{auto_attribs=True}?
-    @param call: AST of the call to L{attr.s()}.
-        This function will assume that L{attr.s()} is called without
-        verifying that.
-    @param module: Module that contains the call, used for error reporting.
-    @return: L{True} if L{True} is passed for C{auto_attribs},
-        L{False} in all other cases: if C{auto_attribs} is not passed,
-        if an explicit L{False} is passed or if an error was reported.
-    """
-    try:
-        args = bind_args(_attrs_decorator_signature, call)
-    except TypeError as ex:
-        message = str(ex).replace("'", '"')
-        module.report(
-            f"Invalid arguments for attr.s(): {message}",
-            lineno_offset=call.lineno
-            )
-        return False
-
-    auto_attribs_expr = args.arguments.get('auto_attribs')
-    if auto_attribs_expr is None:
-        return False
-
-    try:
-        value = ast.literal_eval(auto_attribs_expr)
-    except ValueError:
-        module.report(
-            'Unable to figure out value for "auto_attribs" argument '
-            'to attr.s(), maybe too complex',
-            lineno_offset=call.lineno
-            )
-        return False
-
-    if not isinstance(value, bool):
-        module.report(
-            f'Value for "auto_attribs" argument to attr.s() '
-            f'has type "{type(value).__name__}", expected "bool"',
-            lineno_offset=call.lineno
-            )
-        return False
-
-    return value
-
-
-def is_attrib(expr: Optional[ast.expr], ctx: model.Documentable) -> bool:
-    """Does this expression return an C{attr.ib}?"""
-    return isinstance(expr, ast.Call) and node2fullname(expr.func, ctx) in (
-        'attr.ib', 'attr.attrib', 'attr.attr'
-        )
-
-
-_attrib_signature = signature(attrib)
-"""Signature of the L{attr.ib} function for defining class attributes."""
-
-def attrib_args(expr: ast.expr, ctx: model.Documentable) -> Optional[BoundArguments]:
-    """Get the arguments passed to an C{attr.ib} definition.
-    @return: The arguments, or L{None} if C{expr} does not look like
-        an C{attr.ib} definition or the arguments passed to it are invalid.
-    """
-    if isinstance(expr, ast.Call) and node2fullname(expr.func, ctx) in (
-            'attr.ib', 'attr.attrib', 'attr.attr'
-            ):
-        try:
-            return bind_args(_attrib_signature, expr)
-        except TypeError as ex:
-            message = str(ex).replace("'", '"')
-            ctx.module.report(
-                f"Invalid arguments for attr.ib(): {message}",
-                lineno_offset=expr.lineno
-                )
-    return None
-
-def is_using_typing_final(obj: model.Attribute) -> bool:
-    """
-    Detect if C{obj}'s L{Attribute.annotation} is using L{typing.Final}.
-    """
-    final_qualifiers = ("typing.Final", "typing_extensions.Final")
-    fullName = node2fullname(obj.annotation, obj)
-    if fullName in final_qualifiers:
-        return True
-    if isinstance(obj.annotation, ast.Subscript):
-        # Final[...] or typing.Final[...] expressions
-        if isinstance(obj.annotation.value, (ast.Name, ast.Attribute)):
-            value = obj.annotation.value
-            fullName = node2fullname(value, obj)
-            if fullName in final_qualifiers:
-                return True
-
-    return False
-
 def is_constant(obj: model.Attribute) -> bool:
     """
     Detect if the given assignment is a constant. 
@@ -170,7 +71,52 @@ def is_constant(obj: model.Attribute) -> bool:
     @note: Must be called after setting obj.annotation to detect variables using Final.
     """
 
-    return obj.name.isupper() or is_using_typing_final(obj)
+    return obj.name.isupper() or is_using_typing_final(obj.annotation, obj)
+
+class TypeAliasVisitorExt(extensions.ModuleVisitorExt):
+    """
+    This visitor implements the handling of type aliases and type variables.
+    """
+    def _isTypeVariable(self, ob: model.Attribute) -> bool:
+        if ob.value is not None:
+            if isinstance(ob.value, ast.Call) and node2fullname(ob.value.func, ob) in ('typing.TypeVar', 'typing_extensions.TypeVar'):
+                return True
+        return False
+    
+    def _isTypeAlias(self, ob: model.Attribute) -> bool:
+        """
+        Return C{True} if the Attribute is a type alias.
+        """
+        if ob.value is not None:
+
+            if is_using_annotations(ob.annotation, ('typing.TypeAlias', 'typing_extensions.TypeAlias'), ob):
+                try:
+                    ob.value = unstring_annotation(ob.value, ob)
+                except SyntaxError as e:
+                    ob.report(f"invalid type alias: {e}")
+                    return False
+                return True
+            
+            if is_typing_annotation(ob.value, ob.parent):
+                return True
+        
+        return False
+
+    def visit_Assign(self, node: Union[ast.Assign, ast.AnnAssign]) -> None:
+        current = self.visitor.builder.current
+        for dottedname in iterassign(node): 
+            if dottedname and len(dottedname)==1:
+                attr = current.contents.get(dottedname[0])
+                if attr is None:
+                    return
+                if not isinstance(attr, model.Attribute):
+                    return
+                if self._isTypeAlias(attr) is True:
+                    attr.kind = model.DocumentableKind.TYPE_ALIAS
+                elif self._isTypeVariable(attr) is True:
+                    attr.kind = model.DocumentableKind.TYPE_VARIABLE
+    
+    visit_AnnAssign = visit_Assign
 
 def is_attribute_overridden(obj: model.Attribute, new_value: Optional[ast.expr]) -> bool:
     """
@@ -201,24 +147,14 @@ def extract_final_subscript(annotation: ast.Subscript) -> ast.expr:
         assert isinstance(ann_slice, ast.expr)
         return ann_slice
 
-class ModuleVistor(ast.NodeVisitor):
-    currAttr: Optional[model.Documentable]
-    newAttr: Optional[model.Documentable]
+class ModuleVistor(NodeVisitor):
 
     def __init__(self, builder: 'ASTBuilder', module: model.Module):
+        super().__init__()
         self.builder = builder
         self.system = builder.system
         self.module = module
 
-    def default(self, node: ast.AST) -> None:
-        body: Optional[Sequence[ast.stmt]] = getattr(node, 'body', None)
-        if body is not None:
-            self.currAttr = None
-            for child in body:
-                self.newAttr = None
-                self.visit(child)
-                self.currAttr = self.newAttr
-            self.newAttr = None
 
     def visit_If(self, node: ast.If) -> None:
         if isinstance(node.test, ast.Compare):
@@ -226,8 +162,7 @@ class ModuleVistor(ast.NodeVisitor):
                 # skip if __name__ == '__main__': blocks since
                 # whatever is declared in them cannot be imported
                 # and thus is not part of the API
-                return
-        self.default(node)
+                raise self.SkipNode()
 
     def visit_Module(self, node: ast.Module) -> None:
         assert self.module.docstring is None
@@ -236,32 +171,49 @@ class ModuleVistor(ast.NodeVisitor):
         if len(node.body) > 0 and isinstance(node.body[0], ast.Expr) and isinstance(node.body[0].value, ast.Str):
             self.module.setDocstring(node.body[0].value)
             epydoc2stan.extract_fields(self.module)
-        self.default(node)
+
+    def depart_Module(self, node: ast.Module) -> None:
         self.builder.pop(self.module)
 
-    def visit_ClassDef(self, node: ast.ClassDef) -> Optional[model.Class]:
+    def visit_ClassDef(self, node: ast.ClassDef) -> None:
         # Ignore classes within functions.
         parent = self.builder.current
         if isinstance(parent, model.Function):
-            return None
+            raise self.SkipNode()
 
         rawbases = []
-        bases = []
-        baseobjects = []
+        initialbases = []
+        initialbaseobjects = []
 
-        for n in node.bases:
-            if isinstance(n, ast.Name):
-                str_base = n.id
-            else:
-                str_base = astor.to_source(n).strip()
-
-            rawbases.append(str_base)
-            full_name = parent.expandName(str_base)
-            bases.append(full_name)
-            baseobj = self.system.objForFullName(full_name)
+        for base_node in node.bases:
+            # This handles generics in MRO, by extracting the first
+            # subscript value::
+            #   class Visitor(MyGeneric[T]):...
+            # 'MyGeneric' will be added to rawbases instead 
+            # of 'MyGeneric[T]' which cannot resolve to anything.
+            name_node = base_node
+            if isinstance(base_node, ast.Subscript):
+                name_node = base_node.value
+            
+            str_base = '.'.join(node2dottedname(name_node) or \
+                # Fallback on astor if the expression is unknown by node2dottedname().
+                [astor.to_source(base_node).strip()]) 
+                
+            # Store the base as string and as ast.expr in rawbases list.
+            rawbases += [(str_base, base_node)]
+            
+            # Try to resolve the base, put None if could not resolve it,
+            # if we can't resolve it now, it most likely mean that there are
+            # import cycles (maybe in TYPE_CHECKING blocks). 
+            # None bases will be re-resolved in post-processing.
+            expandbase = parent.expandName(str_base)
+            baseobj = self.system.objForFullName(expandbase)
+            
             if not isinstance(baseobj, model.Class):
                 baseobj = None
-            baseobjects.append(baseobj)
+                
+            initialbases.append(expandbase)
+            initialbaseobjects.append(baseobj)
 
         lineno = node.lineno
         if node.decorator_list:
@@ -270,21 +222,22 @@ class ModuleVistor(ast.NodeVisitor):
         cls: model.Class = self.builder.pushClass(node.name, lineno)
         cls.decorators = []
         cls.rawbases = rawbases
-        cls.bases = bases
-        cls.baseobjects = baseobjects
+        cls._initialbaseobjects = initialbaseobjects
+        cls._initialbases = initialbases
 
         if len(node.body) > 0 and isinstance(node.body[0], ast.Expr) and isinstance(node.body[0].value, ast.Str):
             cls.setDocstring(node.body[0].value)
             epydoc2stan.extract_fields(cls)
 
         if node.decorator_list:
+            
+            cls.raw_decorators = node.decorator_list
+        
             for decnode in node.decorator_list:
                 args: Optional[Sequence[ast.expr]]
                 if isinstance(decnode, ast.Call):
                     base = node2fullname(decnode.func, parent)
                     args = decnode.args
-                    if base in ('attr.s', 'attr.attrs', 'attr.attributes'):
-                        cls.auto_attribs |= _uses_auto_attribs(decnode, parent.module)
                 else:
                     base = node2fullname(decnode, parent)
                     args = None
@@ -295,20 +248,21 @@ class ModuleVistor(ast.NodeVisitor):
                     cls.report("cannot make sense of class decorator")
                 else:
                     cls.decorators.append((base, args))
-        cls.raw_decorators = node.decorator_list if node.decorator_list else []
 
-        for b in cls.baseobjects:
-            if b is not None:
-                b.subclasses.append(cls)
-        self.default(node)
+            
+        # We're not resolving the subclasses at this point yet because all 
+        # modules might not have been processed, and since subclasses are only used in the presentation,
+        # it's better to resolve them in the post-processing instead.
+
+
+    def depart_ClassDef(self, node: ast.ClassDef) -> None:
         self.builder.popClass()
 
-        return cls
 
     def visit_ImportFrom(self, node: ast.ImportFrom) -> None:
         ctx = self.builder.current
         if not isinstance(ctx, model.CanContainImportsDocumentable):
-            self.builder.warning("processing import statement in odd context", str(ctx))
+            # processing import statement in odd context
             return
 
         modname = node.module
@@ -349,10 +303,10 @@ class ModuleVistor(ast.NodeVisitor):
         if mod is None:
             # We don't have any information about the module, so we don't know
             # what names to import.
-            self.builder.warning("import * from unknown", modname)
+            self.builder.current.report(f"import * from unknown {modname}", thresh=1)
             return
 
-        self.builder.warning("import *", modname)
+        self.builder.current.report(f"import * from {modname}", thresh=1)
 
         # Get names to import: use __all__ if available, otherwise take all
         # names that are not private.
@@ -365,13 +319,62 @@ class ModuleVistor(ast.NodeVisitor):
                 if not name.startswith('_')
                 ]
 
+        # Fetch names to export.
+        exports = self._getCurrentModuleExports()
+
         # Add imported names to our module namespace.
         assert isinstance(self.builder.current, model.CanContainImportsDocumentable)
         _localNameToFullName = self.builder.current._localNameToFullName_map
         expandName = mod.expandName
         for name in names:
+
+            if self._handleReExport(exports, name, name, mod) is True:
+                continue
+
             _localNameToFullName[name] = expandName(name)
 
+    def _getCurrentModuleExports(self) -> Collection[str]:
+        # Fetch names to export.
+        current = self.builder.current
+        if isinstance(current, model.Module):
+            exports = current.all
+            if exports is None:
+                exports = []
+        else:
+            # Don't export names imported inside classes or functions.
+            exports = []
+        return exports
+
+    def _handleReExport(self, curr_mod_exports:Collection[str], 
+                        origin_name:str, as_name:str,
+                        origin_module:model.Module) -> bool:
+        """
+        Move re-exported objects into current module.
+
+        @returns: True if the imported name has been sucessfully re-exported.
+        """
+        # Move re-exported objects into current module.
+        current = self.builder.current
+        modname = origin_module.fullName()
+        if as_name in curr_mod_exports:
+            # In case of duplicates names, we can't rely on resolveName,
+            # So we use content.get first to resolve non-alias names. 
+            ob = origin_module.contents.get(origin_name) or origin_module.resolveName(origin_name)
+            if ob is None:
+                current.report("cannot resolve re-exported name :"
+                                        f'{modname}.{origin_name}', thresh=1)
+            else:
+                if origin_module.all is None or origin_name not in origin_module.all:
+                    self.system.msg(
+                        "astbuilder",
+                        "moving %r into %r" % (ob.fullName(), current.fullName())
+                        )
+                    # Must be a Module since the exports is set to an empty list if it's not.
+                    assert isinstance(current, model.Module)
+                    ob.reparent(current, as_name)
+                    return True
+        return False
+
     def _importNames(self, modname: str, names: Iterable[ast.alias]) -> None:
         """Handle a C{from <modname> import <names>} statement."""
 
@@ -379,40 +382,18 @@ class ModuleVistor(ast.NodeVisitor):
         mod = self.system.getProcessedModule(modname)
 
         # Fetch names to export.
-        current = self.builder.current
-        if isinstance(current, model.Module):
-            exports = current.all
-            if exports is None:
-                exports = []
-        else:
-            assert isinstance(current, model.CanContainImportsDocumentable)
-            # Don't export names imported inside classes or functions.
-            exports = []
+        exports = self._getCurrentModuleExports()
 
+        current = self.builder.current
+        assert isinstance(current, model.CanContainImportsDocumentable)
         _localNameToFullName = current._localNameToFullName_map
         for al in names:
             orgname, asname = al.name, al.asname
             if asname is None:
                 asname = orgname
 
-            # Move re-exported objects into current module.
-            if asname in exports and mod is not None:
-                # In case of duplicates names, we can't rely on resolveName,
-                # So we use content.get first to resolve non-alias names. 
-                ob = mod.contents.get(orgname) or mod.resolveName(orgname)
-                if ob is None:
-                    self.builder.warning("cannot resolve re-exported name",
-                                         f'{modname}.{orgname}')
-                else:
-                    if mod.all is None or orgname not in mod.all:
-                        self.system.msg(
-                            "astbuilder",
-                            "moving %r into %r" % (ob.fullName(), current.fullName())
-                            )
-                        # Must be a Module since the exports is set to an empty list if it's not.
-                        assert isinstance(current, model.Module)
-                        ob.reparent(current, asname)
-                        continue
+            if mod is not None and self._handleReExport(exports, orgname, asname, mod) is True:
+                continue
 
             # If we're importing from a package, make sure imported modules
             # are processed (getProcessedModule() ignores non-modules).
@@ -435,8 +416,7 @@ class ModuleVistor(ast.NodeVisitor):
         part of the statement.
         """
         if not isinstance(self.builder.current, model.CanContainImportsDocumentable):
-            self.builder.warning("processing import statement in odd context",
-                                 str(self.builder.current))
+            # processing import statement in odd context
             return
         _localNameToFullName = self.builder.current._localNameToFullName_map
         for al in node.names:
@@ -500,7 +480,7 @@ class ModuleVistor(ast.NodeVisitor):
         obj.kind = model.DocumentableKind.CONSTANT
 
         # A hack to to display variables annotated with Final with the real type instead.
-        if is_using_typing_final(obj):
+        if is_using_typing_final(obj.annotation, obj):
             if isinstance(obj.annotation, ast.Subscript):
                 try:
                     annotation = extract_final_subscript(obj.annotation)
@@ -531,23 +511,35 @@ class ModuleVistor(ast.NodeVisitor):
         if obj is None:
             obj = self.builder.addAttribute(name=target, kind=None, parent=parent)
         
-        if isinstance(obj, model.Attribute):
-            
-            if annotation is None and expr is not None:
-                annotation = _infer_type(expr)
-            
-            obj.annotation = annotation
-            obj.setLineNumber(lineno)
-            
-            if is_constant(obj):
-                self._handleConstant(obj=obj, value=expr, lineno=lineno)
-            else:
-                obj.kind = model.DocumentableKind.VARIABLE
-                # We store the expr value for all Attribute in order to be able to 
-                # check if they have been initialized or not.
-                obj.value = expr
+        # If it's not an attribute it means that the name is already denifed as function/class 
+        # probably meaning that this attribute is a bound callable. 
+        #
+        #   def func(value, stock) -> int:...
+        #   var = 2
+        #   func = partial(func, value=var)
+        #
+        # We don't know how to handle this,
+        # so we ignore it to document the original object. This means that we might document arguments 
+        # that are in reality not existing because they have values in a partial() call for instance.
 
-            self.newAttr = obj
+        if not isinstance(obj, model.Attribute):
+            return
+            
+        if annotation is None and expr is not None:
+            annotation = _infer_type(expr)
+        
+        obj.annotation = annotation
+        obj.setLineNumber(lineno)
+        
+        if is_constant(obj):
+            self._handleConstant(obj=obj, value=expr, lineno=lineno)
+        else:
+            obj.kind = model.DocumentableKind.VARIABLE
+            # We store the expr value for all Attribute in order to be able to 
+            # check if they have been initialized or not.
+            obj.value = expr
+
+        self.builder.currentAttr = obj
 
     def _handleAssignmentInModule(self,
             target: str,
@@ -578,17 +570,9 @@ class ModuleVistor(ast.NodeVisitor):
             obj = self.builder.addAttribute(name=name, kind=None, parent=cls)
 
         if obj.kind is None:
-            instance = is_attrib(expr, cls) or (
-                cls.auto_attribs and annotation is not None and not (
-                    isinstance(annotation, ast.Subscript) and
-                    node2fullname(annotation.value, cls) == 'typing.ClassVar'
-                    )
-                )
-            obj.kind = model.DocumentableKind.INSTANCE_VARIABLE if instance else model.DocumentableKind.CLASS_VARIABLE
+            obj.kind = model.DocumentableKind.CLASS_VARIABLE
 
         if expr is not None:
-            if annotation is None:
-                annotation = self._annotation_from_attrib(expr, cls)
             if annotation is None:
                 annotation = _infer_type(expr)
         
@@ -600,7 +584,7 @@ class ModuleVistor(ast.NodeVisitor):
         else:
             obj.value = expr
 
-        self.newAttr = obj
+        self.builder.currentAttr = obj
 
     def _handleInstanceVar(self,
             name: str,
@@ -617,7 +601,7 @@ class ModuleVistor(ast.NodeVisitor):
         if not _maybeAttribute(cls, name):
             return
 
-        # Class variables can only be Attribute, so it's OK to cast
+        # Class variables can only be Attribute, so it's OK to cast because we used _maybeAttribute() above.
         obj = cast(Optional[model.Attribute], cls.contents.get(name))
         if obj is None:
 
@@ -636,7 +620,8 @@ class ModuleVistor(ast.NodeVisitor):
         else:
             obj.kind = model.DocumentableKind.INSTANCE_VARIABLE
             obj.value = expr
-        self.newAttr = obj
+        
+        self.builder.currentAttr = obj
 
     def _handleAssignmentInClass(self,
             target: str,
@@ -726,7 +711,7 @@ class ModuleVistor(ast.NodeVisitor):
         if type_comment is None:
             annotation = None
         else:
-            annotation = self._unstring_annotation(ast.Str(type_comment, lineno=lineno))
+            annotation = unstring_annotation(ast.Str(type_comment, lineno=lineno), self.builder.current)
 
         for target in node.targets:
             if isinstance(target, ast.Tuple):
@@ -738,18 +723,19 @@ class ModuleVistor(ast.NodeVisitor):
                 self._handleAssignment(target, annotation, expr, lineno)
 
     def visit_AnnAssign(self, node: ast.AnnAssign) -> None:
-        annotation = self._unstring_annotation(node.annotation)
+        annotation = unstring_annotation(node.annotation, self.builder.current)
         self._handleAssignment(node.target, annotation, node.value, node.lineno)
 
     def visit_Expr(self, node: ast.Expr) -> None:
         value = node.value
         if isinstance(value, ast.Str):
-            attr = self.currAttr
+            attr = self.builder.currentAttr
             if attr is not None:
                 attr.setDocstring(value)
-
+                self.builder.currentAttr = None
         self.generic_visit(node)
 
+
     def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> None:
         self._handleFunctionDef(node, is_async=True)
 
@@ -763,18 +749,23 @@ class ModuleVistor(ast.NodeVisitor):
         # Ignore inner functions.
         parent = self.builder.current
         if isinstance(parent, model.Function):
-            return
+            raise self.SkipNode()
 
         lineno = node.lineno
+
+        # setting linenumber from the start of the decorations
         if node.decorator_list:
             lineno = node.decorator_list[0].lineno
 
+        # extracting docstring
         docstring: Optional[ast.Str] = None
         if len(node.body) > 0 and isinstance(node.body[0], ast.Expr) \
                               and isinstance(node.body[0].value, ast.Str):
             docstring = node.body[0].value
 
         func_name = node.name
+
+        # determine the function's kind
         is_property = False
         is_classmethod = False
         is_staticmethod = False
@@ -798,12 +789,13 @@ class ModuleVistor(ast.NodeVisitor):
                     func_name = '.'.join(deco_name[-2:])
 
         if is_property:
+            # handle property and skip child nodes.
             attr = self._handlePropertyDef(node, docstring, lineno)
             if is_classmethod:
                 attr.report(f'{attr.fullName()} is both property and classmethod')
             if is_staticmethod:
                 attr.report(f'{attr.fullName()} is both property and staticmethod')
-            return
+            raise self.SkipNode()
 
         func = self.builder.pushFunction(func_name, lineno)
         func.is_async = is_async
@@ -860,7 +852,11 @@ class ModuleVistor(ast.NodeVisitor):
 
         func.signature = signature
         func.annotations = self._annotations_from_function(node)
-        self.default(node)
+    
+    def depart_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> None:
+        self.builder.popFunction()
+
+    def depart_FunctionDef(self, node: ast.FunctionDef) -> None:
         self.builder.popFunction()
 
     def _handlePropertyDef(self,
@@ -893,31 +889,11 @@ class ModuleVistor(ast.NodeVisitor):
             attr.parsed_docstring = pdoc
 
         if node.returns is not None:
-            attr.annotation = self._unstring_annotation(node.returns)
+            attr.annotation = unstring_annotation(node.returns, attr)
         attr.decorators = node.decorator_list
 
         return attr
 
-    def _annotation_from_attrib(self,
-            expr: ast.expr,
-            ctx: model.Documentable
-            ) -> Optional[ast.expr]:
-        """Get the type of an C{attr.ib} definition.
-        @param expr: The expression's AST.
-        @param ctx: The context in which this expression is evaluated.
-        @return: A type annotation, or None if the expression is not
-                 an C{attr.ib} definition or contains no type information.
-        """
-        args = attrib_args(expr, ctx)
-        if args is not None:
-            typ = args.arguments.get('type')
-            if typ is not None:
-                return self._unstring_annotation(typ)
-            default = args.arguments.get('default')
-            if default is not None:
-                return _infer_type(default)
-        return None
-
     def _annotations_from_function(
             self, func: Union[ast.AsyncFunctionDef, ast.FunctionDef]
             ) -> Mapping[str, Optional[ast.expr]]:
@@ -954,26 +930,10 @@ class ModuleVistor(ast.NodeVisitor):
             # Include parameter names even if they're not annotated, so that
             # we can use the key set to know which parameters exist and warn
             # when non-existing parameters are documented.
-            name: None if value is None else self._unstring_annotation(value)
+            name: None if value is None else unstring_annotation(value, self.builder.current)
             for name, value in _get_all_ast_annotations()
             }
-
-    def _unstring_annotation(self, node: ast.expr) -> ast.expr:
-        """Replace all strings in the given expression by parsed versions.
-        @return: The unstringed node. If parsing fails, an error is logged
-            and the original node is returned.
-        """
-        try:
-            expr = _AnnotationStringParser().visit(node)
-        except SyntaxError as ex:
-            module = self.builder.currentMod
-            assert module is not None
-            module.report(f'syntax error in annotation: {ex}', lineno_offset=node.lineno)
-            return node
-        else:
-            assert isinstance(expr, ast.expr), expr
-            return expr
-
+    
 class _ValueFormatter:
     """
     Class to encapsulate a python value and translate it to HTML when calling L{repr()} on the L{_ValueFormatter}.
@@ -986,7 +946,7 @@ class _ValueFormatter:
         The colorized value as L{ParsedDocstring}.
         """
 
-        self._linker = ctx.docstringlinker
+        self._linker = ctx.docstring_linker
         """
         Linker.
         """
@@ -997,60 +957,10 @@ class _ValueFormatter:
         Without the englobing <code> tags.
         """
         # Using node2stan.node2html instead of flatten(to_stan()). 
-        # This avoids calling flatten() twice.
+        # This avoids calling flatten() twice, 
+        # but potential XML parser errors caused by XMLString needs to be handled later.
         return ''.join(node2stan.node2html(self._colorized.to_node(), self._linker))
 
-class _AnnotationStringParser(ast.NodeTransformer):
-    """Implementation of L{ModuleVistor._unstring_annotation()}.
-
-    When given an expression, the node returned by L{ast.NodeVisitor.visit()}
-    will also be an expression.
-    If any string literal contained in the original expression is either
-    invalid Python or not a singular expression, L{SyntaxError} is raised.
-    """
-
-    def _parse_string(self, value: str) -> ast.expr:
-        statements = ast.parse(value).body
-        if len(statements) != 1:
-            raise SyntaxError("expected expression, found multiple statements")
-        stmt, = statements
-        if isinstance(stmt, ast.Expr):
-            # Expression wrapped in an Expr statement.
-            expr = self.visit(stmt.value)
-            assert isinstance(expr, ast.expr), expr
-            return expr
-        else:
-            raise SyntaxError("expected expression, found statement")
-
-    def visit_Subscript(self, node: ast.Subscript) -> ast.Subscript:
-        value = self.visit(node.value)
-        if isinstance(value, ast.Name) and value.id == 'Literal':
-            # Literal[...] expression; don't unstring the arguments.
-            slice = node.slice
-        elif isinstance(value, ast.Attribute) and value.attr == 'Literal':
-            # typing.Literal[...] expression; don't unstring the arguments.
-            slice = node.slice
-        else:
-            # Other subscript; unstring the slice.
-            slice = self.visit(node.slice)
-        return ast.copy_location(ast.Subscript(value, slice, node.ctx), node)
-
-    # For Python >= 3.8:
-
-    def visit_Constant(self, node: ast.Constant) -> ast.expr:
-        value = node.value
-        if isinstance(value, str):
-            return ast.copy_location(self._parse_string(value), node)
-        else:
-            const = self.generic_visit(node)
-            assert isinstance(const, ast.Constant), const
-            return const
-
-    # For Python < 3.8:
-
-    def visit_Str(self, node: ast.Str) -> ast.expr:
-        return ast.copy_location(self._parse_string(node.s), node)
-
 def _infer_type(expr: ast.expr) -> Optional[ast.expr]:
     """Infer an expression's type.
     @param expr: The expression's AST.
@@ -1106,26 +1016,41 @@ def _annotation_for_elements(sequence: Iterable[object]) -> Optional[ast.expr]:
 DocumentableT = TypeVar('DocumentableT', bound=model.Documentable)
 
 class ASTBuilder:
+    """
+    Keeps tracks of the state of the AST build, creates documentable and adds objects to the system.
+    """
     ModuleVistor = ModuleVistor
 
     def __init__(self, system: model.System):
         self.system = system
-        self.current = cast(model.Documentable, None)
-        self.currentMod: Optional[model.Module] = None
+        
+        self.current = cast(model.Documentable, None) # current visited object
+        self.currentMod: Optional[model.Module] = None # module, set when visiting ast.Module
+        self.currentAttr: Optional[model.Documentable] = None # recently visited attribute object
+        
         self._stack: List[model.Documentable] = []
         self.ast_cache: Dict[Path, Optional[ast.Module]] = {}
 
+
     def _push(self, cls: Type[DocumentableT], name: str, lineno: int) -> DocumentableT:
+        """
+        Create and enter a new object of the given type and add it to the system.
+        """
         obj = cls(self.system, name, self.current)
-        self.system.addObject(obj)
         self.push(obj, lineno)
+        self.system.addObject(obj)
+        self.currentAttr = None
         return obj
 
     def _pop(self, cls: Type[model.Documentable]) -> None:
         assert isinstance(self.current, cls)
         self.pop(self.current)
+        self.currentAttr = None
 
     def push(self, obj: model.Documentable, lineno: int) -> None:
+        """
+        Enter a documentable.
+        """
         self._stack.append(self.current)
         self.current = obj
         if isinstance(obj, model.Module):
@@ -1142,34 +1067,53 @@ class ASTBuilder:
             obj.setLineNumber(lineno)
 
     def pop(self, obj: model.Documentable) -> None:
+        """
+        Leave a documentable.
+        """
         assert self.current is obj, f"{self.current!r} is not {obj!r}"
         self.current = self._stack.pop()
         if isinstance(obj, model.Module):
             self.currentMod = None
 
     def pushClass(self, name: str, lineno: int) -> model.Class:
+        """
+        Create and a new class in the system.
+        """
         return self._push(self.system.Class, name, lineno)
+
     def popClass(self) -> None:
+        """
+        Leave a class.
+        """
         self._pop(self.system.Class)
 
     def pushFunction(self, name: str, lineno: int) -> model.Function:
+        """
+        Create and enter a new function in the system.
+        """
         return self._push(self.system.Function, name, lineno)
+
     def popFunction(self) -> None:
+        """
+        Leave a function.
+        """
         self._pop(self.system.Function)
 
     def addAttribute(self,
             name: str, kind: Optional[model.DocumentableKind], parent: model.Documentable
             ) -> model.Attribute:
+        """
+        Add a new attribute to the system, attributes cannot be "entered".
+        """
         system = self.system
         parentMod = self.currentMod
         attr = system.Attribute(system, name, parent)
         attr.kind = kind
         attr.parentMod = parentMod
         system.addObject(attr)
+        self.currentAttr = attr
         return attr
 
-    def warning(self, message: str, detail: str) -> None:
-        self.system._warning(self.current, message, detail)
 
     def processModuleAST(self, mod_ast: ast.Module, mod: model.Module) -> None:
 
@@ -1181,19 +1125,31 @@ class ASTBuilder:
             else:
                 module_var_parser(node, mod)
 
-        self.ModuleVistor(self, mod).visit(mod_ast)
+        vis = self.ModuleVistor(self, mod)
+        vis.extensions.add(*self.system._astbuilder_visitors)
+        vis.extensions.attach_visitor(vis)
+        vis.walkabout(mod_ast)
 
-    def parseFile(self, path: Path) -> Optional[ast.Module]:
+    def parseFile(self, path: Path, ctx: model.Module) -> Optional[ast.Module]:
         try:
             return self.ast_cache[path]
         except KeyError:
             mod: Optional[ast.Module] = None
             try:
                 mod = parseFile(path)
-            except (SyntaxError, ValueError):
-                self.warning("cannot parse", str(path))
+            except (SyntaxError, ValueError) as e:
+                ctx.report(f"cannot parse file, {e}")
+
             self.ast_cache[path] = mod
             return mod
+    
+    def parseString(self, py_string:str, ctx: model.Module) -> Optional[ast.Module]:
+        mod = None
+        try:
+            mod = _parse(py_string)
+        except (SyntaxError, ValueError):
+            ctx.report("cannot parse string")
+        return mod
 
 model.System.defaultBuilder = ASTBuilder
 
@@ -1287,3 +1243,7 @@ MODULE_VARIABLES_META_PARSERS: Mapping[str, Callable[[ast.Assign, model.Module],
     '__all__': parseAll,
     '__docformat__': parseDocformat
 }
+
+
+def setup_pydoctor_extension(r:extensions.ExtRegistrar) -> None:
+    r.register_astbuilder_visitor(TypeAliasVisitorExt)
