@@ -6,33 +6,33 @@ system being documented.  An instance of L{System} represents the whole system
 being documented -- a System is a bad of Documentables, in some sense.
 """
 
+import abc
 import ast
+from collections import defaultdict
 import datetime
 import importlib
 import inspect
 import platform
 import sys
+import textwrap
 import types
 from enum import Enum
 from inspect import signature, Signature
 from pathlib import Path
 from typing import (
-    TYPE_CHECKING, Any, Collection, Dict, Iterator, List, Mapping,
-    Optional, Sequence, Set, Tuple, Type, TypeVar, overload
+    TYPE_CHECKING, Any, Callable, Collection, Dict, Iterator, List, Mapping,
+    Optional, Sequence, Set, Tuple, Type, TypeVar, Union, cast, overload
 )
-from collections import OrderedDict
 from urllib.parse import quote
 
 from pydoctor.options import Options
-from pydoctor import qnmatch
+from pydoctor import factory, qnmatch, utils, linker, astutils, mro
 from pydoctor.epydoc.markup import ParsedDocstring
 from pydoctor.sphinx import CacheT, SphinxInventory
 
 if TYPE_CHECKING:
     from typing_extensions import Literal
-    from twisted.web.template import Flattenable
-    from pydoctor.astbuilder import ASTBuilder
-    from pydoctor import epydoc2stan
+    from pydoctor.astbuilder import ASTBuilder, DocumentableT
 else:
     Literal = {True: bool, False: bool}
     ASTBuilder = object
@@ -96,11 +96,14 @@ class DocumentableKind(Enum):
     MODULE              = 900
     CLASS               = 800
     INTERFACE           = 850
+    EXCEPTION           = 750
     CLASS_METHOD        = 700
     STATIC_METHOD       = 600
     METHOD              = 500
     FUNCTION            = 400
     CONSTANT            = 310
+    TYPE_VARIABLE       = 306
+    TYPE_ALIAS          = 305
     CLASS_VARIABLE      = 300
     SCHEMA_FIELD        = 220
     ATTRIBUTE           = 210
@@ -141,7 +144,10 @@ class Documentable:
         self.parent = parent
         self.parentMod: Optional[Module] = None
         self.source_path: Optional[Path] = source_path
-        self._deprecated_info: Optional["Flattenable"] = None
+        self.extra_info: List[ParsedDocstring] = []
+        """
+        A list to store extra informations about this documentable, as L{ParsedDocstring}.
+        """
         self.setup()
 
     @property
@@ -150,7 +156,7 @@ class Documentable:
 
     def setup(self) -> None:
         self.contents: Dict[str, Documentable] = {}
-        self._linker: Optional['epydoc2stan.DocstringLinker'] = None
+        self._linker: Optional['linker.DocstringLinker'] = None
 
     def setDocstring(self, node: ast.Str) -> None:
         doc = node.s
@@ -366,12 +372,21 @@ class Documentable:
         assert parentMod is not None
         return parentMod
 
-    def report(self, descr: str, section: str = 'parsing', lineno_offset: int = 0) -> None:
-        """Log an error or warning about this documentable object."""
+    def report(self, descr: str, section: str = 'parsing', lineno_offset: int = 0, thresh:int=-1) -> None:
+        """
+        Log an error or warning about this documentable object.
+
+        @param descr: The error/warning string
+        @param section: What the warning is about.
+        @param lineno_offset: Offset
+        @param thresh: Thresh to pass to L{System.msg}, it will use C{-1} by default, 
+          meaning it will count as a violation and will fail the build if option C{-W} is passed.
+          But this behaviour is not applicable if C{thresh} is greater or equal to zero.
+        """
 
         linenumber: object
         if section in ('docstring', 'resolve_identifier_xref'):
-            linenumber = self.docstring_lineno
+            linenumber = self.docstring_lineno or self.linenumber
         else:
             linenumber = self.linenumber
         if linenumber:
@@ -384,19 +399,17 @@ class Documentable:
         self.system.msg(
             section,
             f'{self.description}:{linenumber}: {descr}',
-            thresh=-1)
+            thresh=thresh)
 
     @property
-    def docstringlinker(self) -> 'epydoc2stan.DocstringLinker':
+    def docstring_linker(self) -> 'linker.DocstringLinker':
         """
-        Returns an instance of L{epydoc2stan.DocstringLinker} suitable for resolving names
+        Returns an instance of L{DocstringLinker} suitable for resolving names
         in the context of the object scope. 
         """
         if self._linker is not None:
             return self._linker
-        # FIXME: avoid cyclic import https://github.com/twisted/pydoctor/issues/507
-        from pydoctor.epydoc2stan import _CachedEpydocLinker
-        self._linker = _CachedEpydocLinker(self)
+        self._linker = linker._CachedEpydocLinker(self)
         return self._linker
 
 
@@ -424,6 +437,8 @@ class Module(CanContainImportsDocumentable):
         """Whether this module is a C-extension."""
         self._py_mod: Optional[types.ModuleType] = None
         """The live module if the module was built from introspection."""
+        self._py_string: Optional[str] = None
+        """The module string if the module was built from text."""
 
         self.all: Optional[Collection[str]] = None
         """Names listed in the C{__all__} variable of this module.
@@ -479,28 +494,174 @@ class Module(CanContainImportsDocumentable):
 class Package(Module):
     kind = DocumentableKind.PACKAGE
 
+# List of exceptions class names in the standard library, Python 3.8.10
+_STD_LIB_EXCEPTIONS = ('ArithmeticError', 'AssertionError', 'AttributeError', 
+    'BaseException', 'BlockingIOError', 'BrokenPipeError', 
+    'BufferError', 'BytesWarning', 'ChildProcessError', 
+    'ConnectionAbortedError', 'ConnectionError', 
+    'ConnectionRefusedError', 'ConnectionResetError', 
+    'DeprecationWarning', 'EOFError', 
+    'EnvironmentError', 'Exception', 'FileExistsError', 
+    'FileNotFoundError', 'FloatingPointError', 'FutureWarning', 
+    'GeneratorExit', 'IOError', 'ImportError', 'ImportWarning', 
+    'IndentationError', 'IndexError', 'InterruptedError', 
+    'IsADirectoryError', 'KeyError', 'KeyboardInterrupt', 'LookupError', 
+    'MemoryError', 'ModuleNotFoundError', 'NameError', 
+    'NotADirectoryError', 'NotImplementedError', 
+    'OSError', 'OverflowError', 'PendingDeprecationWarning', 'PermissionError', 
+    'ProcessLookupError', 'RecursionError', 'ReferenceError', 
+    'ResourceWarning', 'RuntimeError', 'RuntimeWarning', 'StopAsyncIteration', 
+    'StopIteration', 'SyntaxError', 'SyntaxWarning', 'SystemError', 
+    'SystemExit', 'TabError', 'TimeoutError', 'TypeError', 
+    'UnboundLocalError', 'UnicodeDecodeError', 'UnicodeEncodeError', 
+    'UnicodeError', 'UnicodeTranslateError', 'UnicodeWarning', 'UserWarning', 
+    'ValueError', 'Warning', 'ZeroDivisionError')
+def is_exception(cls: 'Class') -> bool:
+    """
+    Whether is class should be considered as 
+    an exception and be marked with the special 
+    kind L{DocumentableKind.EXCEPTION}.
+    """
+    for base in cls.mro(True, False):
+        if base in _STD_LIB_EXCEPTIONS:
+            return True
+    return False
+
+def compute_mro(cls:'Class') -> List[Union['Class', str]]:
+    """
+    Compute the method resolution order for this class.
+    This function will also set the 
+    C{_finalbaseobjects} and C{_finalbases} attributes on 
+    this class and all it's superclasses.
+    """
+    def init_finalbaseobjects(o: 'Class', path:Optional[List['Class']]=None) -> None:
+        if not path:
+            path = []
+        if o in path:
+            cycle_str = " -> ".join([o.fullName() for o in path[path.index(cls):] + [cls]])
+            raise ValueError(f"Cycle found while computing inheritance hierarchy: {cycle_str}")
+        path.append(o)
+        if o._finalbaseobjects is not None:
+            return
+        if o.rawbases:
+            finalbaseobjects: List[Optional[Class]] = []
+            finalbases: List[str] = []
+            for i,((str_base, _), base) in enumerate(zip(o.rawbases, o._initialbaseobjects)):
+                if base:
+                    finalbaseobjects.append(base)
+                    finalbases.append(base.fullName())
+                else:
+                    # Only re-resolve the base object if the base was None.
+                    resolved_base = o.parent.resolveName(str_base)
+                    if isinstance(resolved_base, Class):
+                        base = resolved_base
+                        finalbaseobjects.append(base)
+                        finalbases.append(base.fullName())
+                    else:
+                        # the base object could not be resolved
+                        finalbaseobjects.append(None)
+                        finalbases.append(o._initialbases[i])
+                if base:
+                    # Recurse on super classes
+                    init_finalbaseobjects(base, path.copy())
+            o._finalbaseobjects = finalbaseobjects
+            o._finalbases = finalbases
+    
+    def localbases(o:'Class') -> Iterator[Union['Class', str]]:
+        """
+        Like L{Class.baseobjects} but fallback to the expanded name if the base is not resolved to a L{Class} object.
+        """
+        for s,b in zip(o.bases, o.baseobjects):
+            if isinstance(b, Class):
+                yield b
+            else:
+                yield s
+
+    def getbases(o:Union['Class', str]) -> List[Union['Class', str]]:
+        if isinstance(o, str):
+            return []
+        return list(localbases(o))
+
+    init_finalbaseobjects(cls)
+    return mro.mro(cls, getbases)
 
 class Class(CanContainImportsDocumentable):
     kind = DocumentableKind.CLASS
     parent: CanContainImportsDocumentable
-    bases: List[str]
-    baseobjects: List[Optional['Class']]
     decorators: Sequence[Tuple[str, Optional[Sequence[ast.expr]]]]
-    # Note: While unused in pydoctor itself, raw_decorators is still in use
-    #       by Twisted's custom System class, to find deprecations.
-    raw_decorators: Sequence[ast.expr]
-
-    auto_attribs: bool = False
-    """L{True} iff this class uses the C{auto_attribs} feature of the C{attrs}
-    library to automatically convert annotated fields into attributes.
-    """
+    
+    # set in post-processing:
+    _finalbaseobjects: Optional[List[Optional['Class']]] = None 
+    _finalbases: Optional[List[str]] = None
+    _mro: Optional[List[Union['Class', str]]] = None
 
     def setup(self) -> None:
         super().setup()
-        self.rawbases: List[str] = []
+        self.rawbases: Sequence[Tuple[str, ast.expr]] = []
+        self.raw_decorators: Sequence[ast.expr] = []
         self.subclasses: List[Class] = []
+        self._initialbases: List[str] = []
+        self._initialbaseobjects: List[Optional['Class']] = []
+    
+    def _init_mro(self) -> None:
+        """
+        Compute the correct value of the method resolution order returned by L{mro()}.
+        """
+        try:
+            self._mro = compute_mro(self)
+        except ValueError as e:
+            self.report(str(e), 'mro')
+            self._mro = list(self.allbases(True))
 
+    @overload
+    def mro(self, include_external:'Literal[True]', include_self:bool=True) -> List[Union['Class', str]]:...
+    @overload
+    def mro(self, include_external:'Literal[False]'=False, include_self:bool=True) -> List['Class']:...
+    def mro(self, include_external:bool=False, include_self:bool=True) -> List[Union['Class', str]]: # type:ignore[misc]
+        """
+        Get the method resution order of this class. 
+
+        @note: The actual correct value is only set in post-processing, if L{mro()} is called
+            in the AST visitors, it will return the same as C{list(self.allbases(include_self))}.
+        """
+        if self._mro is None:
+            return list(self.allbases(include_self))
+        
+        _mro: List[Union[str, Class]]
+        if include_external is False:
+            _mro = [o for o in self._mro if not isinstance(o, str)]
+        else:
+            _mro = self._mro
+        if include_self is False:
+            _mro = _mro[1:]
+        return _mro
+
+    @property
+    def bases(self) -> List[str]:
+        """
+        Fully qualified names of the bases of this class.
+        """
+        return self._finalbases if \
+            self._finalbases is not None else self._initialbases
+
+    
+    @property
+    def baseobjects(self) -> List[Optional['Class']]:
+        """
+        Base objects, L{None} value is inserted when the base class could not be found in the system.
+        
+        @note: This property is currently computed two times, a first time when we're visiting the ClassDef and initially creating the object. 
+            It's computed another time in post-processing to try to resolve the names that could not be resolved the first time. This is needed when there are import cycles. 
+            
+            Meaning depending on the state of the system, this property can return either the initial objects or the final objects
+        """
+        return self._finalbaseobjects if \
+            self._finalbaseobjects is not None else self._initialbaseobjects
+    
     def allbases(self, include_self: bool = False) -> Iterator['Class']:
+        """
+        Iterate on all base objects of this class and it's super classes. Doesn't comply with MRO.
+        """
         if include_self:
             yield self
         for b in self.baseobjects:
@@ -512,7 +673,7 @@ class Class(CanContainImportsDocumentable):
 
         @return: the object with the given name, or L{None} if there isn't one
         """
-        for base in self.allbases(include_self=True):
+        for base in self.mro():
             obj: Optional[Documentable] = base.contents.get(name)
             if obj is not None:
                 return obj
@@ -552,7 +713,7 @@ class Inheritable(Documentable):
         yield self
         if not isinstance(self.parent, Class):
             return
-        for b in self.parent.allbases(include_self=False):
+        for b in self.parent.mro(include_self=False):
             if self.name in b.contents:
                 yield b.contents[self.name]
 
@@ -613,7 +774,7 @@ if hasattr(types, "ClassMethodDescriptorType"):
 else:
     func_types += (type(dict.__dict__["fromkeys"]), )
 
-
+_default_extensions = object()
 class System:
     """A collection of related documentable objects.
 
@@ -621,16 +782,30 @@ class System:
     package.
     """
 
-    Class = Class
-    Module = Module
-    Package = Package
-    Function = Function
-    Attribute = Attribute
     # Not assigned here for circularity reasons:
     #defaultBuilder = astbuilder.ASTBuilder
     defaultBuilder: Type[ASTBuilder]
+    systemBuilder: Type['ISystemBuilder']
     options: 'Options'
+    extensions: List[str] = cast('List[str]', _default_extensions)
+    """
+    List of extensions.
 
+    By default, all built-in pydoctor extensions will be loaded.
+    Override this value to cherry-pick extensions. 
+    """
+
+    custom_extensions: List[str] = []
+    """
+    Additional list of extensions to load alongside default extensions.
+    """
+
+    show_attr_value = (DocumentableKind.CONSTANT, 
+                       DocumentableKind.TYPE_VARIABLE, 
+                       DocumentableKind.TYPE_ALIAS)
+    """
+    What kind of attributes we should display the value for?
+    """
 
     def __init__(self, options: Optional['Options'] = None):
         self.allobjects: Dict[str, Documentable] = {}
@@ -651,8 +826,11 @@ class System:
 
         self.projectname = 'my project'
 
-        self.docstring_syntax_errors: Set[str] = set()
-        """FullNames of objects for which the docstring failed to parse."""
+        self.parse_errors: Dict[str, Set[str]] = defaultdict(set)
+        """
+        Dict from the name of the thing we're rendering (C{section}) to the FullNames of objects for which the rendereable elements failed to parse.
+        Typically the renderable element is the C{docstring}, but it can be the decorators, parameter default values or any other colorized AST.
+        """
 
         self.verboselevel = 0
         self.needsnl = False
@@ -660,7 +838,7 @@ class System:
 
         # We're using the id() of the modules as key, and not the fullName becaue modules can
         # be reparented, generating KeyError.
-        self.unprocessed_modules: Dict[int, _ModuleT] = OrderedDict()
+        self.unprocessed_modules: List[_ModuleT] = []
 
         self.module_count = 0
         self.processing_modules: List[str] = []
@@ -671,7 +849,40 @@ class System:
         # We use the fullName of the objets as the dict key in order to bind a full name to a privacy, not an object to a privacy.
         # this way, we are sure the objects' privacy stay true even if we reparent them manually.
         self._privacyClassCache: Dict[str, PrivacyClass] = {}
+        
+        # workaround cyclic import issue
+        from pydoctor import extensions
 
+        # Initialize the extension system
+        self._factory = factory.Factory()
+        self._astbuilder_visitors: List[Type['astutils.NodeVisitorExt']] = []
+        self._post_processors: List[Callable[['System'], None]] = []
+        
+        if self.extensions == _default_extensions:
+            self.extensions = list(extensions.get_extensions())
+        assert isinstance(self.extensions, list)
+        assert isinstance(self.custom_extensions, list)
+        # pydoctor.astbuilder includes some required extensions, so always add it.
+        self.extensions = ['pydoctor.astbuilder'] + self.extensions
+        for ext in self.extensions + self.custom_extensions:
+            # Load extensions
+            extensions.load_extension_module(self, ext)
+
+    @property
+    def Class(self) -> Type['Class']:
+        return self._factory.Class
+    @property
+    def Function(self) -> Type['Function']:
+        return self._factory.Function
+    @property
+    def Module(self) -> Type['Module']:
+        return self._factory.Module
+    @property
+    def Package(self) -> Type['Package']:
+        return self._factory.Package
+    @property
+    def Attribute(self) -> Type['Attribute']:
+        return self._factory.Attribute
 
     @property
     def sourcebase(self) -> Optional[str]:
@@ -713,6 +924,7 @@ class System:
         @param thresh: The minimum verbosity level of the system for this message to actually be printed.
             Meaning passing thresh=-1 will make message still display if C{-q} is passed but not if C{-qq}. 
             Similarly, passing thresh=1 will make the message only apprear if the verbosity level is at least increased once with C{-v}.
+            Using negative thresh will count this message as a violation and will fail the build if option C{-W} is passed.
         @param topthresh: The maximum verbosity level of the system for this message to actually be printed.
         """
         if once:
@@ -770,21 +982,12 @@ class System:
 
         return None
 
-
-    def _warning(self,
-            current: Optional[Documentable],
-            message: str,
-            detail: str
-            ) -> None:
-        if current is not None:
-            fn = current.fullName()
-        else:
-            fn = '<None>'
-        if self.options.verbosity > 0:
-            print(fn, message, detail)
-
-    def objectsOfType(self, cls: Type[T]) -> Iterator[T]:
+    def objectsOfType(self, cls: Union[Type['DocumentableT'], str]) -> Iterator['DocumentableT']:
         """Iterate over all instances of C{cls} present in the system. """
+        if isinstance(cls, str):
+            cls = utils.findClassFromDottedName(cls, 'objectsOfType', 
+                base_class=cast(Type['DocumentableT'], Documentable))
+        assert isinstance(cls, type)
         for o in self.allobjects.values():
             if isinstance(o, cls):
                 yield o
@@ -895,7 +1098,7 @@ class System:
 
     def _addUnprocessedModule(self, mod: _ModuleT) -> None:
         """
-        First add the new module into the unprocessed_modules mapping. 
+        First add the new module into the unprocessed_modules list. 
         Handle eventual duplication of module names, and finally add the 
         module to the system.
         """
@@ -906,7 +1109,7 @@ class System:
             assert isinstance(first, Module)
             self._handleDuplicateModule(first, mod)
         else:
-            self.unprocessed_modules[id(mod)] = mod
+            self.unprocessed_modules.append(mod)
             self.addObject(mod)
             self.progress(
                 "analyzeModule", len(self.allobjects),
@@ -922,7 +1125,7 @@ class System:
             - Packages wins over modules
             - Else, the last added module wins
         """
-        self._warning(dup.parent, "duplicate", str(first))
+        dup.report(f"duplicate {str(first)}", thresh=1)
 
         if first._is_c_module and not isinstance(dup, Package):
             # C-modules wins
@@ -932,7 +1135,8 @@ class System:
             return
         else:
             # Else, the last added module wins
-            del self.unprocessed_modules[id(dup)]
+            self._remove(first)
+            self.unprocessed_modules.remove(first)
             self._addUnprocessedModule(dup)
 
     def _introspectThing(self, thing: object, parent: Documentable, parentMod: _ModuleT) -> None:
@@ -961,8 +1165,6 @@ class System:
                 self.addObject(f)
             elif isinstance(v, type):
                 c = self.Class(self, k, parent)
-                c.bases = []
-                c.baseobjects = []
                 c.rawbases = []
                 c.parentMod = parentMod
                 c.docstring = v.__doc__
@@ -1016,9 +1218,16 @@ class System:
             elif suffix in importlib.machinery.SOURCE_SUFFIXES:
                 self.analyzeModule(path, module_name, package)
             break
+    
+    def _remove(self, o: Documentable) -> None:
+        del self.allobjects[o.fullName()]
+        oc = list(o.contents.values())
+        for c in oc:
+            self._remove(c)
 
     def handleDuplicate(self, obj: Documentable) -> None:
-        '''This is called when we see two objects with the same
+        """
+        This is called when we see two objects with the same
         .fullName(), for example::
 
             class C:
@@ -1030,19 +1239,14 @@ class System:
                         implementation 2
 
         The default is that the second definition "wins".
-        '''
+        """
         i = 0
         fullName = obj.fullName()
         while (fullName + ' ' + str(i)) in self.allobjects:
             i += 1
         prev = self.allobjects[fullName]
-        self._warning(obj.parent, "duplicate", str(prev))
-        def remove(o: Documentable) -> None:
-            del self.allobjects[o.fullName()]
-            oc = list(o.contents.values())
-            for c in oc:
-                remove(c)
-        remove(prev)
+        obj.report(f"duplicate {str(prev)}", thresh=1)
+        self._remove(prev)
         prev.name = obj.name + ' ' + str(i)
         def readd(o: Documentable) -> None:
             self.allobjects[o.fullName()] = o
@@ -1062,14 +1266,16 @@ class System:
         if mod.state is ProcessingState.UNPROCESSED:
             self.processModule(mod)
 
-        assert mod.state in (ProcessingState.PROCESSING, ProcessingState.PROCESSED)
+        assert mod.state in (ProcessingState.PROCESSING, ProcessingState.PROCESSED), mod.state
         return mod
 
     def processModule(self, mod: _ModuleT) -> None:
         assert mod.state is ProcessingState.UNPROCESSED
+        assert mod in self.unprocessed_modules
         mod.state = ProcessingState.PROCESSING
+        self.unprocessed_modules.remove(mod)
         if mod.source_path is None:
-            return
+            assert mod._py_string is not None
         if mod._is_c_module:
             self.processing_modules.append(mod.fullName())
             self.msg("processModule", "processing %s"%(self.processing_modules), 1)
@@ -1079,15 +1285,19 @@ class System:
             assert head == mod.fullName()
         else:
             builder = self.defaultBuilder(self)
-            ast = builder.parseFile(mod.source_path)
+            if mod._py_string is not None:
+                ast = builder.parseString(mod._py_string, mod)
+            else:
+                assert mod.source_path is not None
+                ast = builder.parseFile(mod.source_path, mod)
             if ast:
                 self.processing_modules.append(mod.fullName())
-                self.msg("processModule", "processing %s"%(self.processing_modules), 1)
+                if mod._py_string is None:
+                    self.msg("processModule", "processing %s"%(self.processing_modules), 1)
                 builder.processModuleAST(ast, mod)
                 mod.state = ProcessingState.PROCESSED
                 head = self.processing_modules.pop()
                 assert head == mod.fullName()
-        del self.unprocessed_modules[id(mod)]
         self.progress(
             'process',
             self.module_count - len(self.unprocessed_modules),
@@ -1097,7 +1307,7 @@ class System:
 
     def process(self) -> None:
         while self.unprocessed_modules:
-            mod = next(iter(self.unprocessed_modules.values()))
+            mod = next(iter(self.unprocessed_modules))
             self.processModule(mod)
         self.postProcess()
 
@@ -1109,7 +1319,24 @@ class System:
         without the risk of drawing incorrect conclusions because modules
         were not fully processed yet.
         """
-        pass
+
+        # default post-processing includes:
+        # - Processing of subclasses
+        # - MRO computing.
+        # - Checking whether the class is an exception
+        for cls in self.objectsOfType(Class):
+            
+            cls._init_mro()
+            
+            for b in cls.baseobjects:
+                if b is not None:
+                    b.subclasses.append(cls)
+            
+            if is_exception(cls):
+                cls.kind = DocumentableKind.EXCEPTION
+
+        for post_processor in self._post_processors:
+            post_processor(self)
 
 
     def fetchIntersphinxInventories(self, cache: CacheT) -> None:
@@ -1118,3 +1345,132 @@ class System:
         """
         for url in self.options.intersphinx:
             self.intersphinx.update(cache, url)
+
+class SystemBuildingError(Exception):
+    """
+    Raised when there is a (handled) fatal error while adding modules to the builder.
+    """
+
+class ISystemBuilder(abc.ABC):
+    """
+    Interface class for building a system.
+    """
+    @abc.abstractmethod
+    def __init__(self, system: 'System') -> None:
+        """
+        Create the builder.
+        """
+    @abc.abstractmethod
+    def addModule(self, path: Path, parent_name: Optional[str] = None, ) -> None:
+        """
+        Add a module or package from file system path to the pydoctor system. 
+        If the path points to a directory, adds all submodules recursively.
+
+        @raises SystemBuildingError: If there is an error while adding the module/package.
+        """
+    @abc.abstractmethod
+    def addModuleString(self, text: str, modname: str,
+                        parent_name: Optional[str] = None,
+                        is_package: bool = False, ) -> None:
+        """
+        Add a module from text to the system.
+        """
+    @abc.abstractmethod
+    def buildModules(self) -> None:
+        """
+        Build the modules.
+        """
+
+class SystemBuilder(ISystemBuilder):
+    """
+    This class is only an adapter for some System methods related to module building. 
+    """
+    def __init__(self, system: 'System') -> None:
+        self.system = system
+        self._added: Set[Path] = set()
+
+    def addModule(self, path: Path, parent_name: Optional[str] = None, ) -> None:
+        if path in self._added:
+            return
+        # Path validity check
+        if self.system.options.projectbasedirectory is not None:
+            # Note: Path.is_relative_to() was only added in Python 3.9,
+            #       so we have to use this workaround for now.
+            try:
+                path.relative_to(self.system.options.projectbasedirectory)
+            except ValueError as ex:
+                raise SystemBuildingError(f"Source path lies outside base directory: {ex}")
+        parent: Optional[Package] = None
+        if parent_name:
+            _p = self.system.allobjects[parent_name]
+            assert isinstance(_p, Package)
+            parent = _p
+        if path.is_dir():
+            self.system.msg('addPackage', f"adding directory {path}")
+            if not (path / '__init__.py').is_file():
+                raise SystemBuildingError(f"Source directory lacks __init__.py: {path}")
+            self.system.addPackage(path, parent)
+        elif path.is_file():
+            self.system.msg('addModuleFromPath', f"adding module {path}")
+            self.system.addModuleFromPath(path, parent)
+        elif path.exists():
+            raise SystemBuildingError(f"Source path is neither file nor directory: {path}")
+        else:
+            raise SystemBuildingError(f"Source path does not exist: {path}")
+        self._added.add(path)
+
+    def addModuleString(self, text: str, modname: str,
+                        parent_name: Optional[str] = None,
+                        is_package: bool = False, ) -> None:
+        if parent_name is None:
+            parent = None
+        else:
+            # Set containing package as parent.
+            parent = self.system.allobjects[parent_name]
+            assert isinstance(parent, Package), f"{parent.fullName()} is not a Package, it's a {parent.kind}"
+        
+        factory = self.system.Package if is_package else self.system.Module
+        mod = factory(self.system, name=modname, parent=parent, source_path=None)
+        mod._py_string = textwrap.dedent(text)
+        self.system._addUnprocessedModule(mod)
+
+    def buildModules(self) -> None:
+        self.system.process()
+
+System.systemBuilder = SystemBuilder
+
+def prepend_package(builderT:Type[ISystemBuilder], package:str) -> Type[ISystemBuilder]:
+    """
+    Get a new system builder class, that extends the original C{builder} such that it will always use a "fake" 
+    C{package} to be the only root object of the system and add new modules under it.
+    """
+    
+    class PrependPackageBuidler(builderT): # type:ignore
+        """
+        Support for option C{--prepend-package}.
+        """
+
+        def __init__(self, system: 'System', *, package:str) -> None:
+            super().__init__(system)
+            
+            self.package = package
+            
+            prependedpackage = None
+            for m in package.split('.'):
+                prependedpackage = system.Package(
+                    system, m, prependedpackage)
+                system.addObject(prependedpackage)
+        
+        def addModule(self, path: Path, parent_name: Optional[str] = None, ) -> None:
+            if parent_name is None:
+                parent_name = self.package
+            super().addModule(path, parent_name)
+        
+        def addModuleString(self, text: str, modname: str,
+                            parent_name: Optional[str] = None,
+                            is_package: bool = False, ) -> None:
+            if parent_name is None:
+                parent_name = self.package
+            super().addModuleString(text, modname, parent_name, is_package=is_package)
+    
+    return utils.partialclass(PrependPackageBuidler, package=package)
