@@ -3,7 +3,7 @@ from typing import TYPE_CHECKING, List
 from twisted.web.iweb import ITemplateLoader
 from twisted.web.template import Tag, renderer, tags
 
-from pydoctor.model import Attribute, DocumentableKind
+from pydoctor.model import Attribute
 from pydoctor import epydoc2stan
 from pydoctor.templatewriter import TemplateElement, util
 from pydoctor.templatewriter.pages import format_decorators
@@ -39,9 +39,14 @@ class AttributeChild(TemplateElement):
         return self.ob.fullName()
 
     @renderer
-    def shortFunctionAnchor(self, request: object, tag: Tag) -> "Flattenable":
+    def shortFunctionAnchor(self, request: object, tag: Tag) -> str:
         return self.ob.name
-
+    
+    @renderer
+    def anchorHref(self, request: object, tag: Tag) -> str:
+        name = self.shortFunctionAnchor(request, tag)
+        return f'#{name}'
+    
     @renderer
     def decorator(self, request: object, tag: Tag) -> "Flattenable":
         return list(format_decorators(self.ob))
@@ -71,7 +76,7 @@ class AttributeChild(TemplateElement):
 
     @renderer
     def constantValue(self, request: object, tag: Tag) -> "Flattenable":
-        if self.ob.kind is not DocumentableKind.CONSTANT or self.ob.value is None:
+        if self.ob.kind not in self.ob.system.show_attr_value or self.ob.value is None:
             return tag.clear()
-        # Attribute is a constant (with a value), then display it's value
+        # Attribute is a constant/type alias (with a value), then display it's value
         return epydoc2stan.format_constant_value(self.ob)
