@@ -2,6 +2,9 @@ import ast
 import sys
 from textwrap import dedent
 from typing import Any, Union
+import xml.sax
+
+import pytest
 
 from pydoctor.epydoc.markup._pyval_repr import PyvalColorizer
 from pydoctor.test import NotFoundLinker
@@ -13,6 +16,11 @@ def color(v: Any, linebreakok:bool=True, maxlines:int=5, linelen:int=40) -> str:
     parsed_doc = colorizer.colorize(v)
     return parsed_doc.to_node().pformat() #type: ignore
 
+def colorhtml(v: Any, linebreakok:bool=True, maxlines:int=5, linelen:int=40) -> str:
+    colorizer = PyvalColorizer(linelen=linelen, linebreakok=linebreakok, maxlines=maxlines)
+    parsed_doc = colorizer.colorize(v)
+    return flatten(parsed_doc.to_stan(NotFoundLinker(), compact=True))
+
 def test_simple_types() -> None:
     """
     Integers, floats, None, and complex numbers get printed using str,
@@ -154,7 +162,16 @@ def test_strings() -> None:
     <inline classes="variable-quote">
         '''
 """
-    
+
+def test_non_breaking_spaces() -> None:
+    """
+    This test might fail in the future, when twisted's XMLString supports XHTML entities (see https://github.com/twisted/twisted/issues/11581). 
+    But it will always fail for python 3.6 since twisted dropped support for these versions of python.
+    """
+    with pytest.raises(xml.sax.SAXParseException):
+        colorhtml(ast.parse('"These are non-breaking spaces."').body[0].value) == """""" # type:ignore
+    with pytest.raises(xml.sax.SAXParseException):
+        assert colorhtml("These are non-breaking spaces.") == """"""
     
 def test_strings_quote() -> None:
     """
@@ -1432,11 +1449,11 @@ def test_line_wrapping() -> None:
     <inline classes="variable-ellipsis">
         ...\n"""
 
-def color2(v: Any) -> str:
+def color2(v: Any, linelen:int=50) -> str:
     """
     Pain text colorize.
     """
-    colorizer = PyvalColorizer(linelen=50, maxlines=5)
+    colorizer = PyvalColorizer(linelen=linelen, maxlines=5)
     colorized = colorizer.colorize(v)
     text1 = ''.join(gettext(colorized.to_node()))
     text2 = flatten_text(html2stan(flatten(colorized.to_stan(NotFoundLinker()))))
