@@ -2,30 +2,32 @@
 Unit tests for model.
 """
 
-from inspect import signature
-from optparse import Values
-import os
-from pathlib import Path, PurePosixPath, PureWindowsPath
 import subprocess
-from typing import cast
+import os
+from inspect import signature
+from pathlib import Path, PurePosixPath, PureWindowsPath
+from typing import cast, Optional
 import zlib
 import pytest
 
 from twisted.web.template import Tag
 
+from pydoctor.options import Options
 from pydoctor import model, stanutils
 from pydoctor.templatewriter import pages
-from pydoctor.driver import parse_args
+from pydoctor.utils import parse_privacy_tuple
 from pydoctor.sphinx import CacheT
 from pydoctor.test import CapSys
 from pydoctor.test.test_astbuilder import fromText
+from pydoctor.test.test_packages import processPackage
 
 
 class FakeOptions:
     """
-    A fake options object as if it came from that stupid optparse thing.
+    A fake options object as if it came from argparse.
     """
     sourcehref = None
+    htmlsourcebase: Optional[str] = None
     projectbasedirectory: Path
     docformat = 'epytext'
 
@@ -55,15 +57,53 @@ def test_setSourceHrefOption(projectBaseDir: Path) -> None:
 
     options = FakeOptions()
     options.projectbasedirectory = projectBaseDir
-
-    system = model.System()
-    system.sourcebase = "http://example.org/trac/browser/trunk"
-    system.options = cast(Values, options)
+    options.htmlsourcebase = "http://example.org/trac/browser/trunk"
+    system = model.System(options) # type:ignore[arg-type]
     mod.system = system
     system.setSourceHref(mod, projectBaseDir / "package" / "module.py")
 
     assert mod.sourceHref == "http://example.org/trac/browser/trunk/package/module.py"
 
+def test_htmlsourcetemplate_auto_detect() -> None:
+    """
+    Tests for the recognition of different version control providers
+    that uses differents URL templates to point to line numbers.
+
+    Supported templates are::
+
+        Github : {}#L{lineno}
+        Bitbucket: {}#lines-{lineno}
+        SourceForge : {}#l{lineno}
+    """
+    cases = [
+        ("http://example.org/trac/browser/trunk", 
+         "http://example.org/trac/browser/trunk/pydoctor/test/testpackages/basic/mod.py#L7"),
+
+        ("https://sourceforge.net/p/epydoc/code/HEAD/tree/trunk/epydoc", 
+         "https://sourceforge.net/p/epydoc/code/HEAD/tree/trunk/epydoc/pydoctor/test/testpackages/basic/mod.py#l7"),
+        
+        ("https://bitbucket.org/user/scripts/src/master", 
+         "https://bitbucket.org/user/scripts/src/master/pydoctor/test/testpackages/basic/mod.py#lines-7"),
+    ]
+    for base, var_href in cases:
+        options = model.Options.from_args([f'--html-viewsource-base={base}', '--project-base-dir=.'])
+        system = model.System(options)
+
+        processPackage('basic', systemcls=lambda:system)
+        assert system.allobjects['basic.mod.C'].sourceHref == var_href
+
+def test_htmlsourcetemplate_custom() -> None:
+    """
+    The links to source code web pages can be customized via an CLI argument.
+    """
+    options = model.Options.from_args([
+        '--html-viewsource-base=http://example.org/trac/browser/trunk', 
+        '--project-base-dir=.', 
+        '--html-viewsource-template={mod_source_href}#n{lineno}'])
+    system = model.System(options)
+
+    processPackage('basic', systemcls=lambda:system)
+    assert system.allobjects['basic.mod.C'].sourceHref == "http://example.org/trac/browser/trunk/pydoctor/test/testpackages/basic/mod.py#n7"
 
 def test_initialization_default() -> None:
     """
@@ -80,7 +120,7 @@ def test_initialization_options() -> None:
     """
     Can be initialized with options.
     """
-    options = cast(Values, object())
+    options = Options.defaults()
 
     sut = model.System(options=options)
 
@@ -91,11 +131,11 @@ def test_fetchIntersphinxInventories_empty() -> None:
     """
     Convert option to empty dict.
     """
-    options, _ = parse_args([])
+    options = Options.defaults()
     options.intersphinx = []
     sut = model.System(options=options)
 
-    sut.fetchIntersphinxInventories({})
+    sut.fetchIntersphinxInventories(cast('CacheT', {}))
 
     # Use internal state since I don't know how else to
     # check for SphinxInventory state.
@@ -107,7 +147,7 @@ def test_fetchIntersphinxInventories_content() -> None:
     Download and parse intersphinx inventories for each configured
     intersphix.
     """
-    options, _ = parse_args([])
+    options = Options.defaults()
     options.intersphinx = [
         'http://sphinx/objects.inv',
         'file:///twisted/index.inv',
@@ -299,8 +339,9 @@ def test_c_module_text_signature(capsys:CapSys) -> None:
         system = model.System()
         system.options.introspect_c_modules = True
 
-        system.addPackage(package_path, None)
-        system.process()
+        builder = system.systemBuilder(system)
+        builder.addModule(package_path)
+        builder.buildModules()
         
         assert "Cannot parse signature of mymod.base.invalid_text_signature" in capsys.readouterr().out
         
@@ -361,3 +402,73 @@ def test_resolve_name_subclass(capsys:CapSys) -> None:
         """
     )
     assert m.resolveName('C.v') == m.contents['B'].contents['v']
+
+@pytest.mark.parametrize('privacy', [
+    (['public:m._public**', 'public:m.tests', 'public:m.tests.helpers', 'private:m._public.private', 'hidden:m._public.hidden', 'hidden:m.tests.*']), 
+    (reversed(['private:**private', 'hidden:**hidden', 'public:**_public', 'hidden:m.tests.test**', ])), 
+])
+def test_privacy_switch(privacy:object) -> None:
+    s = model.System()
+    s.options.privacy = [parse_privacy_tuple(p, '--privacy') for p in privacy] # type:ignore
+
+    fromText(
+        """
+        class _public:
+            class _still_public:
+                ...
+            class private:
+                ...
+            class hidden:
+                ...
+
+        class tests(B): # public
+            class helpers: # public
+                ...
+            class test1: # everything else hidden
+                ...
+            class test2:
+                ...
+            class test3:
+                ...
+        """, system=s, modname='m'
+    )
+    allobjs = s.allobjects
+
+    assert allobjs['m._public'].privacyClass == model.PrivacyClass.PUBLIC
+    assert allobjs['m._public._still_public'].privacyClass == model.PrivacyClass.PUBLIC
+    assert allobjs['m._public.private'].privacyClass == model.PrivacyClass.PRIVATE
+    assert allobjs['m._public.hidden'].privacyClass == model.PrivacyClass.HIDDEN
+
+    assert allobjs['m.tests'].privacyClass == model.PrivacyClass.PUBLIC
+    assert allobjs['m.tests.helpers'].privacyClass == model.PrivacyClass.PUBLIC
+    assert allobjs['m.tests.test1'].privacyClass == model.PrivacyClass.HIDDEN
+    assert allobjs['m.tests.test2'].privacyClass == model.PrivacyClass.HIDDEN
+    assert allobjs['m.tests.test3'].privacyClass == model.PrivacyClass.HIDDEN
+
+def test_privacy_reparented() -> None:
+    """
+    Test that the privacy of an object changes if 
+    the name of the object changes (with reparenting).
+    """
+
+    system = model.System()
+
+    mod_private = fromText('''
+    class _MyClass:
+        pass
+    ''', modname='private', system=system)
+
+    mod_export = fromText(
+        'from private import _MyClass # not needed for the test to pass', 
+        modname='public', system=system)
+
+    base = mod_private.contents['_MyClass']
+    assert base.privacyClass == model.PrivacyClass.PRIVATE
+
+    # Manually reparent MyClass
+    base.reparent(mod_export, 'MyClass')
+    assert base.fullName() == 'public.MyClass'
+    assert '_MyClass' not in mod_private.contents
+    assert mod_export.resolveName("MyClass") == base
+
+    assert base.privacyClass == model.PrivacyClass.PUBLIC
