@@ -7,7 +7,7 @@ import tempfile
 import os
 from pathlib import Path, PurePath
 
-from pydoctor import model, templatewriter, stanutils, __version__
+from pydoctor import model, templatewriter, stanutils, __version__, epydoc2stan
 from pydoctor.templatewriter import (FailedToCreateTemplate, StaticTemplate, pages, writer, util,
                                      TemplateLookup, Template, 
                                      HtmlTemplate, UnsupportedTemplateVersion, 
@@ -16,6 +16,7 @@ from pydoctor.templatewriter.pages.table import ChildTable
 from pydoctor.templatewriter.summary import isClassNodePrivate, isPrivate, moduleSummary
 from pydoctor.test.test_astbuilder import fromText
 from pydoctor.test.test_packages import processPackage, testpackages
+from pydoctor.test import CapSys
 from pydoctor.themes import get_themes
 
 if TYPE_CHECKING:
@@ -626,3 +627,113 @@ def test_objects_order_mixed_modules_and_packages() -> None:
 
     assert names == ['aaa', 'aba', 'bbb']
 
+src_crash_xml_entities = '''\
+"""
+These are non-breaking spaces
+=============================
+
+docstring.
+"""
+
+A: Literal['These are non-breaking spaces.'] = True
+
+B = ({}, 'These are non-breaking spaces.')
+
+V = True
+"""
+These are non-breaking spaces.
+"""
+
+@thing('These are non-breaking spaces.')
+def g():
+    ...
+
+def h() -> Literal['These are non-breaking spaces.']:
+    ...
+
+
+def f(a:Literal['These are non-breaking spaces.']='These are non-breaking spaces.') -> int:
+    return {}
+
+def i():
+    """
+    Stuff
+
+    @rtype: V of C
+    """
+    ...
+
+class C(Literal['These are non-breaking spaces.']):
+    ...
+
+'''
+
+@pytest.mark.parametrize('processtypes', [True, False])
+def test_crash_xmlstring_entities(capsys:CapSys, processtypes:bool) -> None:
+    """
+    Crash test for https://github.com/twisted/pydoctor/issues/641
+    
+    This test might fail in the future, when twisted's XMLString supports XHTML entities (see https://github.com/twisted/twisted/issues/11581). 
+    But it will always fail for python 3.6 since twisted dropped support for these versions of python.
+    """
+    system = model.System()
+    system.options.verbosity = -1
+    system.options.processtypes=processtypes
+    mod = fromText(src_crash_xml_entities, system=system, modname='test')
+    for o in mod.system.allobjects.values():
+        epydoc2stan.ensure_parsed_docstring(o)
+    getHTMLOf(mod)
+    getHTMLOf(mod.contents['C'])
+    out = capsys.readouterr().out
+    warnings = '''\
+test:2: bad docstring: SAXParseException: <unknown>:2:25: undefined entity
+test:25: bad signature: SAXParseException: <unknown>:1:88: undefined entity
+test:25: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
+test:17: bad rendering of decorators: SAXParseException: <unknown>:1:102: undefined entity
+test:21: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
+test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity
+test:8: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
+test:10: bad rendering of constant: SAXParseException: <unknown>:1:112: undefined entity
+test:14: bad docstring: SAXParseException: <unknown>:1:13: undefined entity
+test:36: bad rendering of class signature: SAXParseException: <unknown>:1:104: undefined entity
+'''.splitlines()
+    
+    # Some how the type processing get rid of the non breaking spaces, but it's more an implementation
+    # detail rather than a fix for the bug.
+    if processtypes is True:
+        warnings.remove('test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity')
+    
+    assert out == '\n'.join(warnings)+'\n'
+
+@pytest.mark.parametrize('processtypes', [True, False])
+def test_crash_xmlstring_entities_rst(capsys:CapSys, processtypes:bool) -> None:
+    """Idem for RST"""
+    system = model.System()
+    system.options.verbosity = -1
+    system.options.processtypes=processtypes
+    system.options.docformat = 'restructuredtext'
+    mod = fromText(src_crash_xml_entities.replace('@type', ':type').replace('@rtype', ':rtype').replace('==', "--"), modname='test', system=system)
+    for o in mod.system.allobjects.values():
+        epydoc2stan.ensure_parsed_docstring(o)
+    getHTMLOf(mod)
+    getHTMLOf(mod.contents['C'])
+    out = capsys.readouterr().out
+    warn_str = '''\
+test:2: bad docstring: SAXParseException: <unknown>:1:13: undefined entity
+test:25: bad signature: SAXParseException: <unknown>:1:88: undefined entity
+test:25: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
+test:17: bad rendering of decorators: SAXParseException: <unknown>:1:102: undefined entity
+test:21: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
+test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity
+test:8: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
+test:10: bad rendering of constant: SAXParseException: <unknown>:1:112: undefined entity
+test:14: bad docstring: SAXParseException: <unknown>:1:13: undefined entity
+test:36: bad rendering of class signature: SAXParseException: <unknown>:1:104: undefined entity
+'''
+    warnings = warn_str.splitlines()
+
+    if processtypes is True:
+        warnings.remove('test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity')
+    
+    assert out == '\n'.join(warnings)+'\n'
+
